/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#include "keyer_common.h"

// Morse character table
// It is read as follows:
// first octet is 0xF it is a time in half-bits (a dit would be 4 half-bits)
// Otherwise it is read LSB first, 1 is dah and 0 is dit, and shifted right
// until the result is 0x01.
//
// There are a few differences between this table and the Prosign Key
// Assignments used by the K1EL WinKey.  Most of these occur because
// the WinKey has more than one character which sends the same prosign.
// Others are because there is an ITU-R M.1667 recommended character.
//
// Character  Winkey     $10 Keyer           Notes
//  #         EE(null)   Dot space
//  %         EE(null)   Three dot space
//  &         EE(null)   IM
//  *         EE(null)   AA
//  :         KN         OS                  OS is International Morse for :  Use ( for KN
//  ;         AA         KR                  KR is International Morse for ;  Use * for AA
//  <         AR         AK                  Use + for AR
//  >         SK         GM                  Use [ for SK

const uint8_t morse_char[MORSE_CHAR_SIZE] = {
    0xf0,  // 32   word space
    0x2c,  // 33 ! ..--.
    0x52,  // 34 " .-..-.
    0xf2,  // 35 # dot space
    0xc8,  // 36 $ ...-..-
    0xf6,  // 37 % three dot space
    0x1c,  // 38 & ..--
    0x5e,  // 39 ' .----.
    0x2d,  // 40 ( -.--.
    0x6d,  // 41 ) -.--.-
    0x1a,  // 42 * .-.-
    0x2a,  // 43 + .-.-.
    0x73,  // 44 , --..--
    0x61,  // 45 - -....-
    0x6a,  // 46 . .-.-.-
    0x29,  // 47 / -..-.
    0x3f,  // 48 0 -----
    0x3e,  // 49 1 .----
    0x3c,  // 50 2 ..---
    0x38,  // 51 3 ...--
    0x30,  // 52 4 ....-
    0x20,  // 53 5 .....
    0x21,  // 54 6 -....
    0x23,  // 55 7 --...
    0x27,  // 56 8 ---..
    0x2f,  // 57 9 ----.
    0x47,  // 58 : ---...
    0x55,  // 59 ; -.-.-.
    0x36,  // 60 < .--.-
    0x31,  // 61 = -...-
    0x3b,  // 62 > --.--
    0x4c,  // 63 ? ..--..
    0x56,  // 64 @ .--.-.
    0x06,  // 65 A .-
    0x11,  // 66 B -...
    0x15,  // 67 C -.-.
    0x09,  // 68 D -..
    0x02,  // 69 E .
    0x14,  // 70 F ..-.
    0x0b,  // 71 G --.
    0x10,  // 72 H ....
    0x04,  // 73 I ..
    0x1e,  // 74 J .---
    0x0d,  // 75 K -.-
    0x12,  // 76 L .-..
    0x07,  // 77 M --
    0x05,  // 78 N -.
    0x0f,  // 79 O ---
    0x16,  // 80 P .--.
    0x1b,  // 81 Q --.-
    0x0a,  // 82 R .-.
    0x08,  // 83 S ...
    0x03,  // 84 T -
    0x0c,  // 85 U ..-
    0x18,  // 86 V ...-
    0x0e,  // 87 W .--
    0x19,  // 88 X -..-
    0x1d,  // 89 Y -.--
    0x13,  // 90 Z --..
    0x68,  // 91 [ ...-.-
    0x24,  // 92 \ ..-..
    0x22,  // 93 ] .-...
    0x65,  // 94 ^ -.-..-
    0x6c,  // 95 _ ..--.-
    0x17   // 96 ` ---.
};
