/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

// This contains the configuration information.
// All configuration values are unsigned 8 bit values.
// The value names and default values are specified in the
// CONFIGURATION macro which is expanded to produce the
// data structures and the getters and setters.

#ifndef KEYER_CONFIG_H
#define KEYER_CONFIG_H

#include <wirish.h>
#include "keyer_receiver.h"
#include "keyer_version.h"

#define CONFIGURATION                                                                                                                         \
    CV(first,             12,    "FIRS",  NULL,        T_NONE,             0,                0)       /* Must be first                    */  \
    CV(version_maj, KEYER_MAJ,   "VMAJ",  NULL,        T_NONE,             0,                0)       /* Major version                    */  \
    CV(version_min, KEYER_MIN,   "VMIN",  NULL,        T_NONE,             0,                0)       /* Minor version                    */  \
    CV(version_pat, KEYER_PAT,   "VPAT",  NULL,        T_NONE,             0,                0)       /* Patch (bug fix) version          */  \
                                                                                                                                              \
    /* Beep (sidetone) configuration                                                                                                      */  \
    CV(frequency,          63,   "FREQ",  NULL,        T_NONE,    CLASS_BEEP,       MSG_RECALC)       /* Frequency in 10 Hz               */  \
    CV(speaker_vol,        40,   "SVOL",  NULL,        T_NONE,    CLASS_BEEP,       MSG_RECALC)       /* Speaker volume in dB, 0=off      */  \
    CV(headphone_vol,      20,   "HVOL",  NULL,        T_NONE,    CLASS_BEEP,       MSG_RECALC)       /* Headphones volume in dB, 0=off   */  \
    CV(use_phones,          0,   "UPHO",  NULL,        T_NONE,    CLASS_BEEP,       MSG_RECALC)       /* Use headphones if non-zero       */  \
                                                                                                                                              \
    CV(status_speed,       20,   "SSPD", "ST SPEED",   T_SPEED,            0,                0)       /* Status keyer speed in WPM        */  \
    CV(xfrequency,         75,   "XFRQ", "ST FREQ",    T_FREQ,    CLASS_BEEP,       MSG_RECALC)       /* Status frequency in 10 Hz        */  \
    CV(xspeaker_vol,       35,   "XSVL", "ST VOL",     T_VOL,     CLASS_BEEP,       MSG_RECALC)       /* Status speaker volume in dB      */  \
    CV(xheadphone_vol,     20,   "XHVL",  NULL,        T_NONE,    CLASS_BEEP,       MSG_RECALC)       /* Status headphones volume in dB   */  \
    CV(xuse_phones,         0,   "XPHO",  NULL,        T_NONE,    CLASS_BEEP,       MSG_RECALC)       /* Status use headphones            */  \
                                                                                                                                              \
    /* Rotary Encoder configuration                                                                                                       */  \
    CV(encoder_debounce,   10,   "ENCD",  NULL,        T_NONE,             0,                0)       /* Encoder debounce milliseconds    */  \
    CV(button_debounce,    10,   "BTND",  NULL,        T_NONE,             0,                0)       /* Button debounce milliseconds     */  \
    CV(long_press,         10,   "LONG",  NULL,        T_NONE,             0,                0)       /* Long button press *100 ms        */  \
                                                                                                                                              \
     /*  Keyer configuration                                                                                                              */  \
    CV(keyer_type,          1,   "KTYP",  NULL,        T_NONE,   CLASS_KEYER,         MSG_TYPE)       /* Keyer type (enum)                */  \
    CV(keyer_encoder,      32,   "KENC",  NULL,        T_NONE,             0,                0)       /* Keyer encoder speed in WPM       */  \
    CV(keyer_encoder_min,   5,   "KEMN",  NULL,        T_NONE,             0,                0)       /* Keyer encoder minimum speed      */  \
    CV(keyer_encoder_max,  50,   "KEMX",  NULL,        T_NONE,             0,                0)       /* Keyer encoder maximum speed      */  \
    CV(keyer_speed,        32,   "KSPD",  NULL,        T_NONE,   CLASS_KEYER,       MSG_RECALC)       /* Keyer speed in WPM               */  \
    CV(keyer_weight,       50,   "KWGT", "WEIGHT",     T_WEIGHT, CLASS_KEYER,       MSG_RECALC)       /* Keyer weight in percent          */  \
    CV(keyer_local,         1,   "KLCL", "LOCAL",      T_ONOFF,            0,                0)       /* Ignore PC mode, reverse, autosp  */  \
    CV(keyer_paddle_swap,   0,   "KSWP",  NULL,        T_NONE,             0,                0)       /* Left/right paddle swap           */  \
    CV(keyer_autospace,     0,   "KAUT", "AUTOSP",     T_ONOFF,            0,                0)       /* Send space after paddle character */ \
    CV(keyer_farnsworth,    0,   "KFRN",  NULL,        T_NONE,   CLASS_KEYER,       MSG_RECALC)       /* Farnsworth speed (0=disable)     */  \
    CV(keyer_ptt_pre,       3,   "KPRE", "PTT PRE",    T_0255,             0,                0)       /* PTT Pre (tens of milliseconds)   */  \
    CV(keyer_ptt_post,     20,   "KPST", "PTT POST",   T_0255,             0,                0)       /* PTT Post (tens of milliseconds)  */  \
    CV(keyer_space_6,       0,   "KSP6",  NULL,        T_NONE,             0,                0)       /* Six dit (short) word space       */  \
    CV(keyer_default_b,     1,   "KDFB", "DEF B",      T_ONOFF,            0,                0)       /* use B timing for ult, dit, dah   */  \
    CV(keyer_dead_man,     30,   "KDED", "DEAD",       T_0255,             0,                0)       /* Deadman time for tune or key     */  \
                                                                                                                                              \
    /*  MPR121 configuration                                                                                                              */  \
    CV(touch_threshold,    16,   "TOUT",  NULL,        T_NONE,             0,                0)       /* MPR121 touch threshold           */  \
    CV(release_threshold,  10,   "RELT",  NULL,        T_NONE,             0,                0)       /* MPR121 release threshold         */  \
                                                                                                                                              \
    CV(save,                0,   "SAVE", "SAVE",       T_ONOFF,            0,                0)       /* Save current values              */  \
    CV(restore,             0,   "REST", "RESTORE",    T_ONOFF,            0,                0)       /* Restore previously saved values  */  \
    CV(defaults,            0,   "DFLT", "DEFAULT",    T_ONOFF,            0,                0)       /* Reset keyer to default values    */  \
    CV(last,               34,   "LAST", NULL,         T_NONE,             0,                0)       /* Must be last                     */

#define CV(v_name, def_val, ext_name, string_name, set_routine, cls, msg)  uint8_t v_name;

// The configuration data is copied into flash memory.  It is byte-packed to
// save space which decreases the frequency of flash erases.
#pragma pack(push,1)
struct config_data
{
    CONFIGURATION
};
#pragma pack(pop)

#undef CV

static const uint32_t NO_CONFIGURATION = 0xFFFF;

class keyer_config : keyer_receiver
{
  
public:

// Get and set routines for all config parameters
#define CV(v_name, def_val, ext_name, string_name, set_routine, cls, msg)                                  \
    uint8_t get_##v_name() { return m_data.v_name; }        \
    void set_##v_name(uint8_t v) { m_data.v_name = v;       \
                                   notify(offsetof(struct config_data, v_name)); }

    CONFIGURATION

#undef CV

    keyer_config();

    void setup();
    void do_work(uint32_t message, uint32_t data);
    
    uint32_t get_handle(uint8_t *name);
    uint8_t get_by_handle(uint32_t handle);
    void get_name_by_handle(uint32_t handle, uint8_t *buffer);
    void set_by_handle(uint32_t handle, uint8_t val);
    void save();
    void restore();
    void notify();

#ifdef DEBUG
    // For debugging
    void display();
#endif

private:
    struct config_data m_data;

    uint32_t m_param;

    void change_value(uint32_t change);
    void notify(uint32_t param);

    config_data *get_last_good_flash();
};

extern keyer_config config;

#endif    
