/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2010 Michael Hope.
 * Copyright (c) 2012 LeafLabs, LLC.
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#ifndef KEYER_DMA_H
#define KEYER_DMA_H

#include <libmaple/dma.h>

// This routine is identical to dma_setup_transfer, except that this
// one is not deprecated.  The keyer code should probably be converted
// to use the newer tubes but my attempt to do so didn't work.

// This code is hardware dependent.  It depends on the DMA subsystem.

void keyer_dma_setup_transfer(dma_dev       *dev,
                              dma_channel    channel,
                              __io void     *peripheral_address,
                              dma_xfer_size  peripheral_size,
                              __io void     *memory_address,
                              dma_xfer_size  memory_size,
                              uint32         mode);

// The DMA library doesn't have functions to set or clear the circular bit.
static inline void keyer_dma_set_circular(dma_dev *dev, dma_tube tube)
{
    dma_channel_regs(dev, tube)->CCR |= DMA_CIRC_MODE;
}

static inline void keyer_dma_clear_circular(dma_dev *dev, dma_tube tube)
{
    dma_channel_regs(dev, tube)->CCR &= ~DMA_CIRC_MODE;
}

// The DMA library has a dma init function but none to uninit.  This is
// safe to use in the keyer because the sidetone is the only user of DMA.
static inline void keyer_dma_uninit(dma_dev *dev) {
    rcc_clk_disable(dev->clk_id);
}

#endif
