/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#include "keyer_error.h"

// Handle certain types of errors.  None of these should actually
// occur.  The code will send them via the USB serial port.  If
// there is a controlling program it may think they are characters
// and echo them.  Some controlling programs may be confused by
// the output.  Again, these should never appear except while
// developing or debugging the code.

// This class is hardware independent.

void
keyer_error::bad_message(const char *text, uint32_t message, uint32_t data)
//----------------------------------------------------------------------
// Report an object received a message it has no code to handle
//----------------------------------------------------------------------
{
    Serial.print ("Bad internal message from ");
    Serial.print (text);
    Serial.print (": Message 0x");
    Serial.print (message, HEX);
    Serial.print (" Data: 0x");
    Serial.println(data, HEX);
}

void
keyer_error::bad_state(const char *text, uint32_t current_state, uint32_t new_state)
//----------------------------------------------------------------------
// Report tht the keyer has gotten into an unexpected state
//----------------------------------------------------------------------
{
    Serial.print ("Bad keyer state in ");
    Serial.print (text);
    Serial.print (": Current state ");
    Serial.print (current_state, DEC);
    Serial.print (", new state ");
    Serial.println(new_state, DEC);
}

void
keyer_error::general(const char *text)
//----------------------------------------------------------------------
// Report a general error
//----------------------------------------------------------------------
{
    Serial.print ("General error: ");
    Serial.println(text);
}

keyer_error error;
