/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

// This contains the iambic key as well as async ultimatic, dit, and dah

#ifndef KEYER_IAMBIC_H
#define KEYER_IAMBIC_H

#include "keyer_config.h"
#include "keyer_manual.h"

class keyer_iambic : public keyer_manual
{
public:
    keyer_iambic();
    bool setup(keyer_type type, bool silent);
    void paddle_change(uint32_t id, uint32_t data);
    void state_change(keyer_state state);

private:
    bool m_cap_left;
    bool m_cap_right;
    bool m_phy_left;
    bool m_phy_right;

    bool m_idle;

    bool m_type_b;

    bool m_left_pressed_last;
    bool m_left_sent_last;
    bool m_both;

    // These represent the memory latch.  The seen flags indicate
    // that the paddle was pressed.  The OK flags indicate that it
    // is OK to latch the corresponding paddle.
    bool m_left_seen;
    bool m_right_seen;
    bool m_left_ok;
    bool m_right_ok;

    void clear_latch() {
        m_left_seen = false;
        m_right_seen = false;
        m_left_ok = false;
        m_right_ok = false;
    }

    void latch_dit() {
        if (config.get_keyer_paddle_swap()) {
            m_right_ok = true;
        }
        else {
            m_left_ok = true;
        }
    }

    void latch_dah() {
        if (config.get_keyer_paddle_swap()) {
            m_left_ok = true;
        }
        else {
            m_right_ok = true;
        }
    }

    void check_latch() {
        if (m_left_ok && left_paddle()) {
            m_left_seen = true;
        }
        if (m_right_ok && right_paddle()) {
            m_right_seen = true;
        }
    }

    void send_left() {
        if (config.get_keyer_paddle_swap()) {
            keyer.km_send_dah();
        }
        else {
            keyer.km_send_dit();
        }

        m_left_seen = false;
        m_left_ok = false;
        m_left_sent_last = true;
        m_right_ok = true;
    }

    void send_right() {
        if (config.get_keyer_paddle_swap()) {
            keyer.km_send_dit();
        }
        else {
            keyer.km_send_dah();
        }

        m_right_seen = false;
        m_right_ok = false;
        m_left_sent_last = false;
        m_left_ok = true;
    }

    void km_send_dit() {
        keyer.km_send_dit();

        if (config.get_keyer_paddle_swap()) {
            m_right_seen = false;
            m_right_ok = false;
            m_left_sent_last = false;
            m_left_ok = true;
        }
        else {
            m_left_seen = false;
            m_left_ok = false;
            m_left_sent_last = true;
            m_right_ok = true;
        }
    }

    void km_send_dah() {
        keyer.km_send_dah();

        if (config.get_keyer_paddle_swap()) {
            m_left_seen = false;
            m_left_ok = false;
            m_left_sent_last = true;
            m_right_ok = true;         
        }
        else {
            m_right_seen = false;
            m_right_ok = false;
            m_left_sent_last = false;
            m_left_ok = true;
        }
    }

    bool left_paddle() { return m_cap_left || m_phy_left; }
    bool right_paddle() { return m_cap_right || m_phy_right; }
};

extern keyer_iambic iambic;
#endif
