/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

// This contains the main keyer class

#ifndef KEYER_KEYER_H
#define KEYER_KEYER_H

#include <wirish.h>

#include "keyer_common.h"
#include "keyer_config.h"
#include "keyer_manual.h"
#include "keyer_receiver.h"
#include "keyer_types.h"

class keyer_keyer : keyer_receiver
{
public:
    keyer_keyer();
    void setup();
    void do_work(uint32_t message, uint32_t data);

    // These are for the for the keyer_manual subclasses
    void km_send_dit();
    void km_send_dah();
    void km_send_space();
    void km_start_sending();
    void km_stop_sending();
    void km_alter_speed(int32_t delta);
    void km_alter_weight(int32_t delta);
    void km_idle_sync();
    void km_set_echo(bool echo) { m_echo = echo; }

    // These are for the keyer_protocol computer interface
    void pc_keyer_type(keyer_type type);
    void pc_paddle_swap(bool reverse);
    void pc_autospace(bool autospace);
    void pc_space_6(bool space_6);
    void pc_weight(uint32_t weight);
    void pc_ptt_pre(uint32_t ptt_pre);
    void pc_ptt_post(uint32_t ptt_post);
    void pc_sets_speed(bool sets_speed);
    void pc_speed(uint32_t speed);
    void pc_encoder_limits(uint32_t min, uint32_t max);
    void pc_farnsworth(uint32_t farnsworth);
    void pc_abort();
    void pc_tune(bool tune);
    void pc_send(uint8_t character);
    void pc_get_state(keyer_state &state, bool &paddle) { state = m_state; paddle = m_paddle; }
    uint8_t pc_get_encoder_min() { return config.get_keyer_encoder_min(); }
    uint8_t pc_get_encoder_max() { return config.get_keyer_encoder_max(); }
    uint8_t pc_get_encoder() { return config.get_keyer_encoder(); }

private:

    bool m_paddle;

    keyer_manual *m_manual;
    keyer_type m_type;

    keyer_state m_state;
    keyer_state m_next_state;

    int32_t m_alter_speed;
    int32_t m_alter_weight;

    uint32_t m_base_ticks;
    uint32_t m_dit_down_ticks;
    uint32_t m_dah_down_ticks;
    uint32_t m_up_ticks;
    uint32_t m_char_ticks;
    uint32_t m_word_ticks;

    uint32_t m_computer_base_ticks;
    uint32_t m_computer_dit_down_ticks;
    uint32_t m_computer_dah_down_ticks;
    uint32_t m_computer_up_ticks;
    uint32_t m_computer_char_ticks;
    uint32_t m_computer_word_ticks;

    bool m_ptt;
    int32_t m_timer;
    int32_t m_ptt_timer;
    int32_t m_deadman_timer;

    uint32_t m_sequence;
    uint32_t m_ptt_sequence;
    uint32_t m_deadman_sequence;

    uint8_t m_reverse_morse[256];
    uint8_t m_sent;
    uint8_t m_sent_mask;
    bool m_echo;

    uint8_t m_char;
    uint8_t m_bits;

    bool m_computer_sets_speed;
    uint32_t m_computer_speed;

    uint32_t m_sync_idles;
    
    void update_speed();
    void set_ticks(uint32_t ticks);
    void set_ptt_ticks(uint32_t ticks);
    void set_deadman();
    void set_keyer_type(keyer_type type, bool silent);

    void start_paddle(keyer_state state);
    void send_paddle(keyer_state state);
    void paddle_timer();
    void idle_paddle_keyer();

    void start_computer(keyer_state state);
    void stop_computer();
    void send_or_idle_computer();
    void computer_timer();

};

extern keyer_keyer keyer;

#endif
