/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#include "keyer_pins.h"
#include "keyer_error.h"
#include "keyer_messages.h"
#include "keyer_outputs.h"
#include "keyer_queue.h"

// The keyer supports a key and a PTT output.  This receives on and off
// messages and sets the output pins.
//
// This class is hardware dependent.  It depends on the I/O subsystem.
// Much of this is abstracted away by the Arduino library.

keyer_outputs::keyer_outputs()
{
    pinMode(KEY1_OUT, OUTPUT);
    pinMode(PTT1_OUT, OUTPUT);
}

void
keyer_outputs::setup()
//----------------------------------------------------------------------
// Set up the PTT and key output instance
//----------------------------------------------------------------------
{
    queue.subscribe(CLASS_OUTPUTS, this);
}

void
keyer_outputs::do_work(uint32_t message, uint32_t data)
//----------------------------------------------------------------------
// Handle a PTT and key output change messages
//----------------------------------------------------------------------
{
    switch (MESSAGE_ID(message)) {

        case MSG_OUTPUT_CHANGED:
        {
            if (data == OUTPUT_OFF) {
                digitalWrite(KEY1_OUT, LOW);
            }
            else {
                digitalWrite(KEY1_OUT, HIGH);
            }
            break;
        }

        case MSG_PTT_CHANGED:
        {
            if (data == OUTPUT_OFF) {
                digitalWrite(PTT1_OUT, LOW);
                digitalWrite(LED, HIGH);
            }
            else {
                digitalWrite(PTT1_OUT, HIGH);
                digitalWrite(LED, LOW);
            }
            break;
        }

        default:
            error.bad_message("Outputs", message, data);
    }
}

keyer_outputs outputs;
