/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#include "keyer_pins.h"
#include "keyer_messages.h"
#include "keyer_paddle.h"
#include "keyer_queue.h"

// This class interfaces with a physical paddle.  When a paddle is pressed
// or released an interrupt is generated.  This code sends a message to the
// keyer when that happens.

// This class is hardware dependent.  It depends on the I/O and interrupt
// subsystems.  Some of this is abstracted by the Arduino library.

keyer_paddle::keyer_paddle()
{
}

void
keyer_paddle::setup()
//----------------------------------------------------------------------
// Set up the physical paddle
//----------------------------------------------------------------------
{
    pinMode(PADDLE_L, INPUT_PULLUP);
    pinMode(PADDLE_R, INPUT_PULLUP);
    attachInterrupt(PADDLE_L, keyer_paddle::left_interrupt, CHANGE);
    attachInterrupt(PADDLE_R, keyer_paddle::right_interrupt, CHANGE);
}

void
keyer_paddle::left_interrupt()
//----------------------------------------------------------------------
// Handle a left physical paddle change
//----------------------------------------------------------------------
{
    if (digitalRead(PADDLE_L)) {
        queue.put(MESSAGE(CLASS_KEYER, MSG_PHY_LEFT_CHANGED), PADDLE_OPEN);
    }
    else {
        queue.put(MESSAGE(CLASS_KEYER, MSG_PHY_LEFT_CHANGED), PADDLE_CLOSED);
    }
}

void keyer_paddle::right_interrupt()
//----------------------------------------------------------------------
// Handle a right physical paddle change
//----------------------------------------------------------------------
{     
    if (digitalRead(PADDLE_R)) {
        queue.put(MESSAGE(CLASS_KEYER, MSG_PHY_RIGHT_CHANGED), PADDLE_OPEN);
    }
    else {
        queue.put(MESSAGE(CLASS_KEYER, MSG_PHY_RIGHT_CHANGED), PADDLE_CLOSED);
    }
}

keyer_paddle paddle;
