/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#include <wirish.h>

#include "keyer_keyer.h"
#include "keyer_misc.h"
#include "keyer_queue.h"
#include "keyer_straight_key.h"

// This implements the manual straight key emulation.

// This provides the manual (paddle) control for the keyer as a straight
// key.  The code in keyer_keyer calls this through the public routines
// inherited from keyer_manual.  And this code calls the keyer through
// the public routines in that code.
//
// This class is hardware indepent.

keyer_straight_key::keyer_straight_key()
{
    
}

bool
keyer_straight_key::setup(keyer_type type, bool silent)
//----------------------------------------------------------------------
// If the keyer needs a straight key set up to handle it
//----------------------------------------------------------------------
{
    if (type != KEYER_TYPE_STRAIGHT_KEY) {
        return false;
    }

    if (!silent) {
        queue.put(MESSAGE(CLASS_STATUS, MSG_STATUS_TEXT), "SK");
    }

    m_cap_left = false;
    m_cap_right = false;
    m_phy_left = false;

    m_sending = false;

    return true;
}

void
keyer_straight_key::paddle_change(uint32_t id, uint32_t data)
//----------------------------------------------------------------------
// Handle a paddle opening or closing
//----------------------------------------------------------------------
{
    switch (id) {
    case MSG_CAP_LEFT_CHANGED:
        {
            m_cap_left = (data != 0);
            break;
        }

        case MSG_CAP_RIGHT_CHANGED:
        {
            m_cap_right = (data != 0);
            break;
        }

        case MSG_PHY_LEFT_CHANGED:
        {
            m_phy_left = (data != 0);
            break;
        }

        case MSG_PHY_RIGHT_CHANGED:
        {
            // This is the ring on the stereo connector.  If a mono
            // phone plug is inserted it will be shorted to ground.
            // Ignore it for straight key.
            break;
        }
    }

    bool sending = m_cap_left || m_cap_right || m_phy_left;
    if (sending != m_sending) {
        if (sending) {
            keyer.km_start_sending();
        }
        else {
            keyer.km_stop_sending();
        }
        m_sending = sending;
    }
}

void
keyer_straight_key::state_change(keyer_state state)
//----------------------------------------------------------------------
// Handle keyer state changes
//----------------------------------------------------------------------
{
    UNUSED(state);
}

keyer_straight_key straight_key;
