/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#ifndef KEYER_TIMER_H
#define KEYER_TIMER_H

// The Arduino / Maple library does not have routines for setting a timer
// to fire only once or to count down.  These routines provide the missing
// capabilities.

// This code is hardware dependent. It depends on the timer subsystem.

/**
 * @brief Set the timer to only fire once (not repeat)
 *
 * Does not affect the timer's mode or other settings.
 *
 * @param dev Device to be set into one pulse mode
 */
static inline void keyer_timer_oneshot(timer_dev *dev) {
    *bb_perip(&(dev->regs).bas->CR1, TIMER_CR1_OPM_BIT) = 1;
}

/**
 * @brief Set the counter to count down.
 *
 * Does not affect the timer's mode or other settings.
 *
 * @param dev Device to set into count down mode.
 */
static inline void keyer_timer_count_down(timer_dev *dev) {
    *bb_perip(&(dev->regs).bas->CR1, TIMER_CR1_DIR_BIT) = 1;
}

/**
 * @brief Clear the interrupt bit.
 *
 * Does not affect the timer's mode or other settings.
 *
 * @param dev Device to set into count down mode.
 */
static inline void keyer_timer_clear_interrupt(timer_dev *dev) {
    (dev->regs).bas->SR &= ~TIMER_SR_UIF;
}


#endif
