/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

#ifndef KEYER_TYPES_H
#define KEYER_TYPES_H

// The keyer types and states, used by the main and status keyers

// The manual keyer types
enum keyer_type {
    KEYER_TYPE_FIRST = 0,       // First type for increment/decrement
    KEYER_TYPE_IAMBIC_A = 0,    // Both paddles alternate, no last dit/dah
    KEYER_TYPE_IAMBIC_B,        // Both paddles alternate, last dit/dah
    KEYER_TYPE_ULTIMATIC,       // Both paddles send last pressed
    KEYER_TYPE_REAL_ULTIMATIC,  // Synchronous Ultimatic
    KEYER_TYPE_DIT,             // Both paddles send dit
    KEYER_TYPE_DAH,             // Both paddles send dah
    KEYER_TYPE_BUG,             // Dits send from paddle with decay, non-completing, dahs are straight key
    KEYER_TYPE_STRAIGHT_KEY,    // Straight key
    KEYER_TYPE_FENCE            // Must be last
};

enum keyer_state {
    KEYER_STATE_IDLE,           // Idle
    KEYER_STATE_IDLE_SYNC,      // Idle with clock running
    KEYER_STATE_PTT_PRE,        // PTT asserted waiting to send
    KEYER_STATE_DOWN_DIT,       // Sending dit
    KEYER_STATE_DOWN_DAH,       // Sending dah
    KEYER_STATE_DOWN,           // Sending untimed (straight key or bug)
    KEYER_STATE_UP,             // Dit or dah key up
    KEYER_STATE_SPACE,          // Arbitrary length space (from computer)
    KEYER_STATE_CHAR_SPACE,     // Inter-character space
    KEYER_STATE_PTT,            // PTT asserted
    KEYER_STATE_TUNE,           // Key down for tune
};

#endif
