/******************************************************************************
 * The MIT License
 *
 * Copyright (c) 2016, 2017 Paul Young
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *****************************************************************************/

// This contains the synchronous ultimatic key

#ifndef KEYER_ULTIMATIC_H
#define KEYER_ULTIMATIC_H

#include "keyer_config.h"
#include "keyer_manual.h"

class keyer_ultimatic : public keyer_manual
{
public:
    keyer_ultimatic();
    bool setup(keyer_type type, bool silent);
    void paddle_change(uint32_t id, uint32_t data);
    void state_change(keyer_state state);

private:
    bool m_cap_left;
    bool m_cap_right;
    bool m_phy_left;
    bool m_phy_right;

    bool m_left_pressed_last;

    // These represent the memory latch.
    bool m_left_seen;
    bool m_right_seen;

    void send_left() {
        if (config.get_keyer_paddle_swap()) {
            keyer.km_send_dah();
        }
        else {
            keyer.km_send_dit();
        }

        m_left_seen = false;
    }

    void send_right() {
        if (config.get_keyer_paddle_swap()) {
            keyer.km_send_dit();
        }
        else {
            keyer.km_send_dah();
        }

        m_right_seen = false;
    }

    bool left_paddle() { return m_cap_left || m_phy_left; }
    bool right_paddle() { return m_cap_right || m_phy_right; }
};

extern keyer_ultimatic ultimatic;
#endif
